      subroutine set_vpr30_params( zmf )

      implicit none

      include 'vpr30.h'

      real zmf

c   *** Constants.  Grav is the gravitational acceleration in m/s^2.

      pi = acos(-1.0)
      tan3 = tan(3.0*pi/180.0)
      one_third = 1.0/3.0
      grav = 9.81
      
c   *** Runge-Kutt time-stepping parameters.  Currently RK-2.

      nrk_max = 2
      wt_rk1(1) =  1.0
      wt_rk2(1) =  0.0
      wt_rk1(2) =  0.5
      wt_rk2(2) = -0.5

c   *** R_c is the circulation computation radius in m.

      R_c = 15.0

c   *** Halo vortex positions normalized by b.  These are intended to 
c   *** remain fixed.

      yhv(1) = -0.35
      zhv(1) =  0.90
      yhv(2) = -1.05
      zhv(2) =  0.00
      yhv(3) = -0.40
      zhv(3) = -0.55

      yhv(4) = -yhv(3)
      zhv(4) =  zhv(3)
      yhv(5) = -yhv(2)
      zhv(5) =  zhv(2)
      yhv(6) = -yhv(1)
      zhv(6) =  zhv(1)

c   *** Partition factors for baroclinic induced circulation.  There has
c   *** not been a need to adjust these values.

      baro_part(1) = 0.20
      baro_part(2) = 0.65
      baro_part(3) = 1.0 - baro_part(1) - baro_part(2)
      baro_part(4) = baro_part(3)
      baro_part(5) = baro_part(2)
      baro_part(6) = baro_part(1)

c   *** Halo vortex circulation partition factors.  1-initial circulation
c   *** partition value, 2-final circulation partition value, 3-time value
c   *** for middle of transition, 4-time scale for transition.  Only the
c   *** final circulation partition values may be tuned, the other values
c   *** are intended to remain fixed.  Adjust (Gamma_oo)2 according to
c   *** 1.39017*[0.865962-(Gamma_oo)1]-0.203837 in order to produce a
c   *** pure downward motion of the vortex system as t-->oo.  Reduce
c   *** (Gamma_oo)1 to increase the downward velocity of the vortex pair.
c   *** Increase (Gamma_oo)2 from the curve fit above to add a positive
c   *** separation velocity for the pair.  You probably should not use
c   *** a negative separation velocity.

      halo_fac(1,1) = 0.45
      halo_fac(1,2) = 0.608     !tunable
      halo_fac(1,3) = 1.16
      halo_fac(1,4) = 1.24
      halo_fac(2,1) = 0.35
      halo_fac(2,2) = 0.155	!tunable
      halo_fac(2,3) = 1.74
      halo_fac(2,4) = 2.07

c   *** Vortex annihilation circulation fraction and time scale.

      A_ann =  0.80     ! should be fixed (or vary with R_c/R0)
      B_ann = -0.05     ! tunable
      C_ann =  0.002    ! tunable

c   *** Primary circulation decay (normalized by b0 and v0)

      tau =  10.0	! tunable
      t_switch = 3.0    ! tunable

c   *** Adjustments for turbulence

      tau_min = 7.0	!tunable
      tau_rate = 50.0	!tunable

c   *** Shear gradient parameters

      t_roll = 0.5	! tunable

      c_sg1 = 0.35	! tunable
      c_sg2 = 0.35	! tunable
      
      n_fit = 1
      sg_fac(1,1) = -0.64       ! tunable
      sg_fac(2,1) =  0.23       ! tunable
      sg_fac(5,1) =  2.86       ! tunable
      sg_fac(6,1) = -0.57       ! tunable
      sg_fac(3,1) =  1.0 - sg_fac(1,1) - sg_fac(2,1)
      sg_fac(4,1) =  1.0 - sg_fac(5,1) - sg_fac(6,1)

      data iflip/6,5,4,3,2,1/

c   *** Z_ige_fac is the altitude (normalized by b0) where we switch to ige.

      z_ige_fac = zmf

      return
      end
